<?php

namespace Drupal\entity_translation_reminder\Form;

use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Entity\ContentEntityTypeInterface;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Entity translation reminder configuration form.
 *
 * This form is heavily inspired by Drupal core ContentLanguageSettingsForm.
 */
class EntityTranslationReminderConfigForm extends ConfigFormBase implements ContainerInjectionInterface {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * The entity bundle info.
   *
   * @var \Drupal\Core\Entity\EntityTypeBundleInfoInterface
   */
  protected EntityTypeBundleInfoInterface $entityTypeBundleInfo;

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['entity_translation_reminder.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'entity_translation_reminder_settings_form';
  }

  /**
   * Constructs an \Drupal\views\Plugin\views\argument_validator\Entity object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Entity\EntityTypeBundleInfoInterface $entity_type_bundle_info
   *   The entity type bundle info.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, EntityTypeBundleInfoInterface $entity_type_bundle_info) {
    $this->entityTypeManager = $entity_type_manager;
    $this->entityTypeBundleInfo = $entity_type_bundle_info;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('entity_type.bundle.info')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('entity_translation_reminder.settings');
    $labels = [];
    $entity_types = $this->entityTypeManager->getDefinitions();
    $bundles = $this->entityTypeBundleInfo->getAllBundleInfo();
    foreach ($entity_types as $entity_type_id => $entity_type) {
      if (!$entity_type instanceof ContentEntityTypeInterface || !$entity_type->hasKey('langcode') || !isset($bundles[$entity_type_id]) || !$entity_type->isTranslatable()) {
        continue;
      }
      $labels[$entity_type_id] = $entity_type->getLabel() ?: $entity_type_id;
    }

    asort($labels);

    // Convert boolean values to a workable values array for the FAPI.
    $enabled_types = array_keys(array_filter($config->get('entity_types') ?: []));

    $form['entity_types'] = [
      '#title' => $this->t('Enabled entity types'),
      '#description' => $this->t('Only entity types which are configured to be translatable are shown.'),
      '#type' => 'checkboxes',
      '#options' => $labels,
      '#default_value' => $enabled_types ?: [],
    ];

    $form['settings'] = ['#tree' => TRUE];

    foreach ($labels as $entity_type_id => $label) {
      $entity_type = $entity_types[$entity_type_id];

      $form['settings'][$entity_type_id] = [
        '#title' => $label,
        '#type' => 'details',
        '#entity_type' => $entity_type_id,
        '#bundle_label' => $entity_type->getBundleLabel() ?: $label,
        '#states' => [
          'visible' => [
            ':input[name="entity_types[' . $entity_type_id . ']"]' => ['checked' => TRUE],
          ],
        ],
      ];

      $bundles_added = FALSE;
      foreach ($bundles[$entity_type_id] as $bundle => $bundle_info) {
        // Skip if bundle is not translatable.
        if (!$bundle_info['translatable']) {
          continue;
        }

        $form['settings'][$entity_type_id][$bundle] = [
          '#type' => 'checkbox',
          '#title' => $bundle_info['label'],
          '#default_value' => $config->get('settings.' . $entity_type_id . '.' . $bundle . '') ?: FALSE,
        ];
        $bundles_added = TRUE;
      }

      // Show a message if there are no translatable bundles.
      if (!$bundles_added) {
        $form['settings'][$entity_type_id]['empty'] = [
          '#markup' => $this->t('No translatable bundles found for %type', ['%type' => $label]),
        ];
      }
    }

    $form['actions']['#type'] = 'actions';
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save configuration'),
      '#button_type' => 'primary',
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);
    $config = $this->config('entity_translation_reminder.settings');
    $config->set('entity_types', $form_state->getValue('entity_types'));
    $config->set('settings', $form_state->getValue('settings'));
    $config->save();
  }

}
