<?php

namespace Drupal\entity_translation_reminder\Service;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;

/**
 * Service for handling translation reminders.
 */
class ReminderService {

  use StringTranslationTrait;

  /**
   * The messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected MessengerInterface $messenger;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected ConfigFactoryInterface $configFactory;

  /**
   * Constructs a new ReminderService.
   *
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   */
  public function __construct(MessengerInterface $messenger, ConfigFactoryInterface $config_factory) {
    $this->messenger = $messenger;
    $this->configFactory = $config_factory;
  }

  /**
   * Check entity settings and show reminder if needed.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity being saved.
   */
  public function checkEntityAndShowReminder(EntityInterface $entity): void {
    // Only proceed if entity is translatable.
    if (!method_exists($entity, 'isTranslatable') || !$entity->isTranslatable()) {
      return;
    }

    // Check if this entity has translation, if not, return.
    if (!method_exists($entity, 'getTranslationLanguages')) {
      return;
    }
    $translation_languages = $entity->getTranslationLanguages(FALSE);
    if (count($translation_languages) === 0) {
      return;
    }

    // Check if this entity type is enabled for translation reminders.
    $entity_type_id = $entity->getEntityTypeId();
    $bundle = $entity->bundle();
    $config = $this->configFactory->get('entity_translation_reminder.settings');
    $enabled_entity_types = $config->get('entity_types') ?: [];
    if (empty($enabled_entity_types[$entity_type_id])) {
      return;
    }

    // Check if this bundle is enabled.
    $bundle_enabled = $config->get('settings.' . $entity_type_id . '.' . $bundle);
    if (!$bundle_enabled) {
      return;
    }

    // Display the message.
    $title_arg = $entity->label();
    if (method_exists($entity, 'toLink')) {
      $title_arg = $entity->toLink()->toString();
    }

    // Display the reminder message.
    $this->messenger->addWarning($this->t('Remember to update all translations for %title.', [
      '%title' => $title_arg,
    ]));
  }

}
